@@ -6,6 +6,7 @@
 #define TREMOTESF_LOG_FORMATTERS_H
 
 #include <concepts>
+#include <cstdint>
 #include <stdexcept>
 #include <system_error>
 #include <type_traits>
@@ -18,12 +19,10 @@
 #    include <QUtf8StringView>
 #endif
 
-#include <fmt/core.h>
-#if FMT_VERSION < 80000
-#    include <fmt/format.h>
-#    define FORMAT_CONST
+#if FMT_VERSION_MAJOR >= 11
+#    include <fmt/base.h>
 #else
-#    define FORMAT_CONST const
+#    include <fmt/core.h>
 #endif
 
 namespace tremotesf {
@@ -34,37 +33,37 @@ namespace tremotesf {
 
 template<>
 struct fmt::formatter<QString> : formatter<string_view> {
-    format_context::iterator format(const QString& string, format_context& ctx) FORMAT_CONST;
+    format_context::iterator format(const QString& string, format_context& ctx) const;
 };
 
 class QStringView;
 template<>
 struct fmt::formatter<QStringView> : formatter<string_view> {
-    format_context::iterator format(const QStringView& string, format_context& ctx) FORMAT_CONST;
+    format_context::iterator format(const QStringView& string, format_context& ctx) const;
 };
 
 class QLatin1String;
 template<>
 struct fmt::formatter<QLatin1String> : formatter<string_view> {
-    format_context::iterator format(const QLatin1String& string, format_context& ctx) FORMAT_CONST;
+    format_context::iterator format(const QLatin1String& string, format_context& ctx) const;
 };
 
 class QByteArray;
 template<>
 struct fmt::formatter<QByteArray> : formatter<string_view> {
-    format_context::iterator format(const QByteArray& array, format_context& ctx) FORMAT_CONST;
+    format_context::iterator format(const QByteArray& array, format_context& ctx) const;
 };
 
 #if QT_VERSION_MAJOR >= 6
 template<>
 struct fmt::formatter<QUtf8StringView> : formatter<string_view> {
-    format_context::iterator format(const QUtf8StringView& string, format_context& ctx) FORMAT_CONST;
+    format_context::iterator format(const QUtf8StringView& string, format_context& ctx) const;
 };
 
 class QAnyStringView;
 template<>
 struct fmt::formatter<QAnyStringView> : formatter<QString> {
-    format_context::iterator format(const QAnyStringView& string, format_context& ctx) FORMAT_CONST;
+    format_context::iterator format(const QAnyStringView& string, format_context& ctx) const;
 };
 #endif
 
@@ -76,7 +75,7 @@ namespace tremotesf::impl {
 
     template<QDebugPrintable T>
     struct QDebugFormatter : SimpleFormatter {
-        fmt::format_context::iterator format(const T& t, fmt::format_context& ctx) FORMAT_CONST {
+        fmt::format_context::iterator format(const T& t, fmt::format_context& ctx) const {
             QString buffer{};
             QDebug stream(&buffer);
             stream.nospace() << t;
@@ -84,35 +83,36 @@ namespace tremotesf::impl {
         }
     };
 
-    fmt::format_context::iterator formatQEnum(const QMetaEnum& meta, qint64 value, fmt::format_context& ctx);
-    fmt::format_context::iterator formatQEnum(const QMetaEnum& meta, quint64 value, fmt::format_context& ctx);
-
+    // This relies on private Qt API but it should work at least until Qt 7
     template<typename T>
-        requires std::is_enum_v<T>
-    struct QEnumFormatter : SimpleFormatter {
-        fmt::format_context::iterator format(T t, fmt::format_context& ctx) FORMAT_CONST {
-            const auto meta = QMetaEnum::fromType<T>();
-            using UnderlyingType = std::underlying_type_t<T>;
-            const auto underlying = static_cast<UnderlyingType>(t);
-            if constexpr (std::signed_integral<UnderlyingType>) {
-                return formatQEnum(meta, static_cast<qint64>(underlying), ctx);
-            } else {
-                return formatQEnum(meta, static_cast<quint64>(underlying), ctx);
-            }
-        }
-    };
+    concept QEnum = std::is_enum_v<T> && requires { qt_getEnumMetaObject(T{}); };
+
+    fmt::format_context::iterator formatQEnum(const QMetaEnum& meta, std::intmax_t value, fmt::format_context& ctx);
+    fmt::format_context::iterator formatQEnum(const QMetaEnum& meta, std::uintmax_t value, fmt::format_context& ctx);
 }
 
 namespace fmt {
     template<std::derived_from<QObject> T>
     struct formatter<T> : tremotesf::SimpleFormatter {
-        format_context::iterator format(const T& object, format_context& ctx) FORMAT_CONST {
+        format_context::iterator format(const T& object, format_context& ctx) const {
             QString buffer{};
             QDebug stream(&buffer);
             stream.nospace() << &object;
             return fmt::format_to(ctx.out(), tremotesf::impl::singleArgumentFormatString, buffer);
         }
     };
+
+    template<tremotesf::impl::QEnum T>
+    struct formatter<T> : tremotesf::SimpleFormatter {
+        fmt::format_context::iterator format(T t, fmt::format_context& ctx) const {
+            const auto meta = QMetaEnum::fromType<T>();
+            if constexpr (std::signed_integral<std::underlying_type_t<T>>) {
+                return tremotesf::impl::formatQEnum(meta, static_cast<std::intmax_t>(t), ctx);
+            } else {
+                return tremotesf::impl::formatQEnum(meta, static_cast<std::uintmax_t>(t), ctx);
+            }
+        }
+    };
 }
 
 #define SPECIALIZE_FORMATTER_FOR_QDEBUG(Class)                                \
@@ -121,16 +121,10 @@ namespace fmt {
         struct formatter<Class> : tremotesf::impl::QDebugFormatter<Class> {}; \
     }
 
-#define SPECIALIZE_FORMATTER_FOR_Q_ENUM(Enum)                              \
-    namespace fmt {                                                        \
-        template<>                                                         \
-        struct formatter<Enum> : tremotesf::impl::QEnumFormatter<Enum> {}; \
-    }
-
 namespace fmt {
     template<>
     struct formatter<std::exception> : tremotesf::SimpleFormatter {
-        format_context::iterator format(const std::exception& e, format_context& ctx) FORMAT_CONST;
+        format_context::iterator format(const std::exception& e, format_context& ctx) const;
     };
 
     template<std::derived_from<std::exception> T>
@@ -139,7 +133,7 @@ namespace fmt {
 
     template<>
     struct formatter<std::system_error> : tremotesf::SimpleFormatter {
-        format_context::iterator format(const std::system_error& e, format_context& ctx) FORMAT_CONST;
+        format_context::iterator format(const std::system_error& e, format_context& ctx) const;
     };
 
     template<std::derived_from<std::system_error> T>
@@ -155,12 +149,12 @@ namespace winrt {
 namespace fmt {
     template<>
     struct formatter<winrt::hstring> : formatter<QString> {
-        format_context::iterator format(const winrt::hstring& str, format_context& ctx) FORMAT_CONST;
+        format_context::iterator format(const winrt::hstring& str, format_context& ctx) const;
     };
 
     template<>
     struct formatter<winrt::hresult_error> : tremotesf::SimpleFormatter {
-        format_context::iterator format(const winrt::hresult_error& e, format_context& ctx) FORMAT_CONST;
+        format_context::iterator format(const winrt::hresult_error& e, format_context& ctx) const;
     };
 }
 #endif
