@@ -13,13 +13,18 @@
 #include <QByteArray>
 #include <QObject>
 
-#include "log/formatters.h"
-#include "serversettings.h"
-#include "serverstats.h"
+#include "coroutines/scope.h"
 #include "torrent.h"
 
-class QFile;
-class QTimer;
+#ifdef Q_MOC_RUN
+#    include "serversettings.h"
+#    include "serverstats.h"
+#else
+namespace tremotesf {
+    class ServerSettings;
+    class ServerStats;
+}
+#endif
 
 namespace tremotesf {
     Q_NAMESPACE
@@ -31,7 +36,7 @@ namespace tremotesf {
     struct ConnectionConfiguration {
         Q_GADGET
     public:
-        enum class ProxyType { Default, Http, Socks5 };
+        enum class ProxyType { Default, Http, Socks5, None };
         Q_ENUM(ProxyType)
 
         QString address{};
@@ -115,9 +120,6 @@ namespace tremotesf {
 
         int torrentsCount() const;
 
-        bool isUpdateDisabled() const;
-        void setUpdateDisabled(bool disabled);
-
         void setConnectionConfiguration(const ConnectionConfiguration& configuration);
         void resetConnectionConfiguration();
 
@@ -125,29 +127,18 @@ namespace tremotesf {
         void disconnect();
 
         void addTorrentFile(
-            const QString& filePath,
-            const QString& downloadDirectory,
-            const std::vector<int>& unwantedFiles,
-            const std::vector<int>& highPriorityFiles,
-            const std::vector<int>& lowPriorityFiles,
-            const std::map<QString, QString>& renamedFiles,
+            QString filePath,
+            QString downloadDirectory,
+            std::vector<int> unwantedFiles,
+            std::vector<int> highPriorityFiles,
+            std::vector<int> lowPriorityFiles,
+            std::map<QString, QString> renamedFiles,
             TorrentData::Priority bandwidthPriority,
             bool start
         );
 
-        void addTorrentFile(
-            std::shared_ptr<QFile> file,
-            const QString& downloadDirectory,
-            const std::vector<int>& unwantedFiles,
-            const std::vector<int>& highPriorityFiles,
-            const std::vector<int>& lowPriorityFiles,
-            const std::map<QString, QString>& renamedFiles,
-            TorrentData::Priority bandwidthPriority,
-            bool start
-        );
-
-        void addTorrentLink(
-            const QString& link, const QString& downloadDirectory, TorrentData::Priority bandwidthPriority, bool start
+        void addTorrentLinks(
+            QStringList links, QString downloadDirectory, TorrentData::Priority bandwidthPriority, bool start
         );
 
         void startTorrents(std::span<const int> ids);
@@ -162,55 +153,74 @@ namespace tremotesf {
 
         void reannounceTorrents(std::span<const int> ids);
 
-        void setSessionProperty(const QString& property, const QJsonValue& value);
-        void setSessionProperties(const QJsonObject& properties);
-        void
-        setTorrentProperty(int id, const QString& property, const QJsonValue& value, bool updateIfSuccessful = false);
-        void setTorrentsLocation(std::span<const int> ids, const QString& location, bool moveFiles);
-        void getTorrentsFiles(std::span<const int> ids, bool asDataUpdate);
-        void getTorrentsPeers(std::span<const int> ids, bool asDataUpdate);
+        void setSessionProperty(QString property, QJsonValue value);
+        void setSessionProperties(QJsonObject properties);
+        void setTorrentProperty(int id, QString property, QJsonValue value, bool updateIfSuccessful = false);
+        void setTorrentsLocation(std::span<const int> ids, QString location, bool moveFiles);
+        void getTorrentFiles(int torrentId);
+        void getTorrentPeers(int torrentId);
 
-        void renameTorrentFile(int torrentId, const QString& filePath, const QString& newName);
+        void renameTorrentFile(int torrentId, QString filePath, QString newName);
 
-        void getDownloadDirFreeSpace();
-        void getFreeSpaceForPath(const QString& path);
-
-        void updateData();
+        Coroutine<std::optional<qint64>> getDownloadDirFreeSpace();
+        Coroutine<std::optional<qint64>> getFreeSpaceForPath(QString path);
 
         void shutdownServer();
 
     private:
         void setStatus(Status&& status);
-        void resetStateOnConnectionStateChanged(ConnectionState oldConnectionState, size_t& removedTorrentsCount);
-        void emitSignalsOnConnectionStateChanged(ConnectionState oldConnectionState, size_t removedTorrentsCount);
+        void resetStateOnConnectionStateChanged(ConnectionState oldConnectionState);
+        void emitSignalsOnConnectionStateChanged(ConnectionState oldConnectionState);
 
-        void getServerSettings();
-        void getTorrents();
-        void checkTorrentsSingleFile(std::span<const int> torrentIds);
-        void getServerStats();
+        Coroutine<> postRequest(QLatin1String method, QJsonObject arguments, bool updateIfSuccessful = true);
 
-        bool checkIfUpdateCompleted();
-        bool checkIfConnectionCompleted();
-        void maybeFinishUpdateOrConnection();
+        Coroutine<> addTorrentFileImpl(
+            QString filePath,
+            QString downloadDirectory,
+            std::vector<int> unwantedFiles,
+            std::vector<int> highPriorityFiles,
+            std::vector<int> lowPriorityFiles,
+            std::map<QString, QString> renamedFiles,
+            TorrentData::Priority bandwidthPriority,
+            bool start
+        );
+        Coroutine<> addTorrentLinksImpl(
+            QStringList links, QString downloadDirectory, TorrentData::Priority bandwidthPriority, bool start
+        );
+        Coroutine<> addTorrentLinkImpl(QString link, QString downloadDirectory, int bandwidthPriority, bool start);
+        Coroutine<> getTorrentsFiles(QJsonArray ids);
+        Coroutine<> getTorrentsPeers(QJsonArray ids);
+        Coroutine<> renameTorrentFileImpl(int torrentId, QString filePath, QString newName);
+        Coroutine<std::optional<qint64>> getDownloadDirFreeSpaceImpl();
+        Coroutine<std::optional<qint64>> getFreeSpaceForPathImpl(QString path);
+        Coroutine<> shutdownServerImpl();
 
-        void checkIfServerIsLocal();
+        Coroutine<> getServerSettings();
+        Coroutine<> getTorrents();
+        Coroutine<> checkTorrentsSingleFile(std::vector<int> torrentIds);
+        Coroutine<> getServerStats();
+
+        Coroutine<> connectAndPerformDataUpdates();
+        Coroutine<> updateData();
+
+        Coroutine<> checkIfServerIsLocal();
+
+        void onRequestFailed(RpcError error, const QString& errorMessage, const QString& detailedErrorMessage);
+        Coroutine<> autoReconnect();
 
         impl::RequestRouter* mRequestRouter{};
-
-        bool mUpdateDisabled{};
-        bool mUpdating{};
+        CoroutineScope mBackgroundRequestsCoroutineScope{};
+        CoroutineScope mAutoReconnectCoroutineScope{};
 
         bool mAutoReconnectEnabled{};
 
         std::optional<bool> mServerIsLocal{};
-        std::optional<int> mPendingHostInfoLookupId{};
 
-        QTimer* mUpdateTimer{};
-        QTimer* mAutoReconnectTimer{};
+        std::chrono::seconds mUpdateInterval{};
+        std::chrono::seconds mAutoReconnectInterval{};
 
         ServerSettings* mServerSettings{};
-        // Don't use member initializer to workaround Android NDK bug (https://github.com/android/ndk/issues/1798)
-        std::vector<std::unique_ptr<Torrent>> mTorrents;
+        std::vector<std::unique_ptr<Torrent>> mTorrents{};
         ServerStats* mServerStats{};
 
         Status mStatus{};
@@ -220,7 +230,6 @@ namespace tremotesf {
         void statusChanged();
         void connectedChanged();
         void connectionStateChanged();
-        void errorChanged();
 
         void onAboutToRemoveTorrents(size_t first, size_t last);
         void onRemovedTorrents(size_t first, size_t last);
@@ -228,36 +237,14 @@ namespace tremotesf {
         void onAboutToAddTorrents(size_t count);
         void onAddedTorrents(size_t count);
 
-        void torrentsUpdated(
-            const std::vector<std::pair<int, int>>& removedIndexRanges,
-            const std::vector<std::pair<int, int>>& changedIndexRanges,
-            int addedCount
-        );
-
-        void torrentFilesUpdated(const tremotesf::Torrent* torrent, const std::vector<int>& changedIndexes);
-        void torrentPeersUpdated(
-            const tremotesf::Torrent* torrent,
-            const std::vector<std::pair<int, int>>& removedIndexRanges,
-            const std::vector<std::pair<int, int>>& changedIndexRanges,
-            int addedCount
-        );
-
-        void torrentFileRenamed(int torrentId, const QString& filePath, const QString& newName);
+        void torrentsUpdated();
 
         void torrentAdded(tremotesf::Torrent* torrent);
         void torrentFinished(tremotesf::Torrent* torrent);
 
         void torrentAddDuplicate();
-        void torrentAddError();
-
-        void gotDownloadDirFreeSpace(qint64 bytes);
-        void gotFreeSpaceForPath(const QString& path, bool success, qint64 bytes);
-
-        void updateDisabledChanged();
+        void torrentAddError(const QString& filePathOrUrl);
     };
 }
 
-SPECIALIZE_FORMATTER_FOR_Q_ENUM(tremotesf::RpcConnectionState)
-SPECIALIZE_FORMATTER_FOR_Q_ENUM(tremotesf::RpcError)
-
 #endif // TREMOTESF_RPC_RPC_H
