@@ -9,7 +9,6 @@
 #include <stdexcept>
 
 #include <QCheckBox>
-#include <QClipboard>
 #include <QComboBox>
 #include <QDialogButtonBox>
 #include <QFileInfo>
@@ -19,23 +18,28 @@
 #include <QLabel>
 #include <QLineEdit>
 #include <QMessageBox>
+#include <QPlainTextEdit>
 #include <QPushButton>
 #include <QTimer>
 #include <QVBoxLayout>
 
 #include <KMessageWidget>
 
+#include <fmt/format.h>
+
 #include "addtorrenthelpers.h"
+#include "formatutils.h"
+#include "magnetlinkparser.h"
 #include "settings.h"
 #include "stdutils.h"
-#include "formatutils.h"
+#include "torrentfileparser.h"
+#include "coroutines/threadpool.h"
 #include "log/log.h"
 #include "rpc/torrent.h"
 #include "rpc/rpc.h"
 #include "ui/widgets/torrentremotedirectoryselectionwidget.h"
 #include "ui/widgets/torrentfilesview.h"
 
-#include "droppedtorrents.h"
 #include "localtorrentfilesmodel.h"
 
 namespace tremotesf {
@@ -62,62 +66,63 @@ namespace tremotesf {
         }
     }
 
-    AddTorrentDialog::AddTorrentDialog(Rpc* rpc, const QString& url, Mode mode, QWidget* parent)
+    AddTorrentDialog::AddTorrentDialog(Rpc* rpc, std::variant<FileParams, UrlParams> params, QWidget* parent)
         : QDialog(parent),
           mRpc(rpc),
-          mUrl(url),
-          mMode(mode),
-          mFilesModel(mode == Mode::File ? new LocalTorrentFilesModel(this) : nullptr) {
+          mParams(std::move(params)),
+          mFilesModel(isAddingFile() ? new LocalTorrentFilesModel(this) : nullptr) {
         setupUi();
 
-        if (mFilesModel) {
-            mFilesModel->load(url);
-        }
-
         QSize size(448, 0);
-        if (mMode == Mode::File) {
+        if (isAddingFile()) {
             size.setHeight(512);
         }
         resize(sizeHint().expandedTo(size));
+
+        if (isAddingFile()) {
+            mParseTorrentFileCoroutineScope.launch(parseTorrentFile());
+        }
     }
 
     void AddTorrentDialog::accept() {
-        if (mMode == Mode::File) {
-            mRpc->addTorrentFile(
-                mUrl,
-                mAddTorrentParametersWidgets.downloadDirectoryWidget->path(),
-                mFilesModel->unwantedFiles(),
-                mFilesModel->highPriorityFiles(),
-                mFilesModel->lowPriorityFiles(),
-                mFilesModel->renamedFiles(),
-                priorityFromComboBoxIndex(mAddTorrentParametersWidgets.priorityComboBox->currentIndex()),
-                mAddTorrentParametersWidgets.startTorrentCheckBox->isChecked()
-            );
+        if (isAddingFile()) {
+            if (!checkIfTorrentFileExists()) {
+                mRpc->addTorrentFile(
+                    std::get<FileParams>(mParams).filePath,
+                    mAddTorrentParametersWidgets.downloadDirectoryWidget->path(),
+                    mFilesModel->unwantedFiles(),
+                    mFilesModel->highPriorityFiles(),
+                    mFilesModel->lowPriorityFiles(),
+                    mFilesModel->renamedFiles(),
+                    priorityFromComboBoxIndex(mAddTorrentParametersWidgets.priorityComboBox->currentIndex()),
+                    mAddTorrentParametersWidgets.startTorrentCheckBox->isChecked()
+                );
+            }
         } else {
-            mRpc->addTorrentLink(
-                mTorrentLinkLineEdit->text(),
-                mAddTorrentParametersWidgets.downloadDirectoryWidget->path(),
-                priorityFromComboBoxIndex(mAddTorrentParametersWidgets.priorityComboBox->currentIndex()),
-                mAddTorrentParametersWidgets.startTorrentCheckBox->isChecked()
-            );
+            QStringList urls = mTorrentLinkTextField->toPlainText().split('\n', Qt::SkipEmptyParts);
+            parseMagnetLinksAndCheckIfTorrentsExist(urls);
+            if (!urls.empty()) {
+                mRpc->addTorrentLinks(
+                    std::move(urls),
+                    mAddTorrentParametersWidgets.downloadDirectoryWidget->path(),
+                    priorityFromComboBoxIndex(mAddTorrentParametersWidgets.priorityComboBox->currentIndex()),
+                    mAddTorrentParametersWidgets.startTorrentCheckBox->isChecked()
+                );
+            }
         }
-
         const auto settings = Settings::instance();
-        if (settings->rememberOpenTorrentDir() && mMode == Mode::File) {
-            settings->setLastOpenTorrentDirectory(QFileInfo(mUrl).path());
+        if (settings->rememberOpenTorrentDir() && isAddingFile()) {
+            settings->setLastOpenTorrentDirectory(QFileInfo(std::get<FileParams>(mParams).filePath).path());
         }
         if (settings->rememberAddTorrentParameters()) {
             mAddTorrentParametersWidgets.saveToSettings();
         }
-        if (mMode == Mode::File && mAddTorrentParametersWidgets.deleteTorrentFileGroupBox->isChecked()) {
-            deleteTorrentFile(mUrl, mAddTorrentParametersWidgets.moveTorrentFileToTrashCheckBox->isChecked());
-        }
-
+        deleteTorrentFileIfEnabled();
         QDialog::accept();
     }
 
     AddTorrentDialog::AddTorrentParametersWidgets
-    AddTorrentDialog::createAddTorrentParametersWidgets(Mode mode, QFormLayout* layout, Rpc* rpc) {
+    AddTorrentDialog::createAddTorrentParametersWidgets(bool forTorrentFile, QFormLayout* layout, Rpc* rpc) {
         const auto parameters = getAddTorrentParameters(rpc);
 
         auto* const downloadDirectoryWidget = new TorrentDownloadDirectoryDirectorySelectionWidget{};
@@ -155,7 +160,7 @@ namespace tremotesf {
 
         QGroupBox* deleteTorrentFileGroupBox{};
         QCheckBox* moveTorrentFileToTrashCheckBox{};
-        if (mode == Mode::File) {
+        if (forTorrentFile) {
             deleteTorrentFileGroupBox = new QGroupBox(qApp->translate("tremotesf", "Delete .torrent file"));
             layout->addWidget(deleteTorrentFileGroupBox);
             deleteTorrentFileGroupBox->setCheckable(true);
@@ -177,8 +182,10 @@ namespace tremotesf {
         };
     }
 
+    bool AddTorrentDialog::isAddingFile() const { return std::holds_alternative<FileParams>(mParams); }
+
     void AddTorrentDialog::setupUi() {
-        if (mMode == Mode::File) {
+        if (isAddingFile()) {
             //: Dialog title
             setWindowTitle(qApp->translate("tremotesf", "Add Torrent File"));
         } else {
@@ -189,95 +196,48 @@ namespace tremotesf {
         auto layout = new QFormLayout(this);
         layout->setSizeConstraint(QLayout::SetMinAndMaxSize);
 
-        auto messageWidget = new KMessageWidget(this);
-        messageWidget->setCloseButtonVisible(false);
-        messageWidget->hide();
-        layout->addRow(messageWidget);
+        mMessageWidget = new KMessageWidget(this);
+        mMessageWidget->setCloseButtonVisible(false);
+        mMessageWidget->hide();
+        layout->addRow(mMessageWidget);
 
-        mTorrentLinkLineEdit = new QLineEdit(mUrl, this);
-        if (mMode == Mode::File) {
-            mTorrentLinkLineEdit->setReadOnly(true);
+        if (isAddingFile()) {
+            mTorrentFilePathTextField = new QLineEdit(std::get<FileParams>(mParams).filePath, this);
+            mTorrentFilePathTextField->setReadOnly(true);
             //: Input field's label
-            layout->addRow(qApp->translate("tremotesf", "Torrent file:"), mTorrentLinkLineEdit);
+            layout->addRow(qApp->translate("tremotesf", "Torrent file:"), mTorrentFilePathTextField);
         } else {
+            mTorrentLinkTextField = new QPlainTextEdit(std::get<UrlParams>(mParams).urls.join('\n'), this);
+            mTorrentLinkTextField->textCursor().setPosition(0);
             //: Input field's label
-            layout->addRow(qApp->translate("tremotesf", "Torrent link:"), mTorrentLinkLineEdit);
-            if (mUrl.isEmpty() && Settings::instance()->fillTorrentLinkFromClipboard()) {
-                const auto dropped = DroppedTorrents(QGuiApplication::clipboard()->mimeData());
-                if (!dropped.urls.isEmpty()) {
-                    logInfo("AddTorrentDialog: filling torrent link from clipboard = {}", dropped.urls.first());
-                    mTorrentLinkLineEdit->setText(dropped.urls.first());
-                }
-            }
-            if (!mTorrentLinkLineEdit->text().isEmpty()) {
-                mTorrentLinkLineEdit->setCursorPosition(0);
-            }
-            QObject::connect(mTorrentLinkLineEdit, &QLineEdit::textChanged, this, &AddTorrentDialog::canAcceptUpdate);
+            layout->addRow(qApp->translate("tremotesf", "Torrent link:"), mTorrentLinkTextField);
+            QObject::connect(
+                mTorrentLinkTextField,
+                &QPlainTextEdit::textChanged,
+                this,
+                &AddTorrentDialog::canAcceptUpdate
+            );
         }
 
-        mAddTorrentParametersWidgets = createAddTorrentParametersWidgets(mMode, layout, mRpc);
+        mAddTorrentParametersWidgets = createAddTorrentParametersWidgets(isAddingFile(), layout, mRpc);
 
-        auto freeSpaceLabel = new QLabel(this);
-        auto getFreeSpace = [=, this](const QString& directory) {
-            if (!directory.isEmpty()) {
-                if (mRpc->serverSettings()->data().canShowFreeSpaceForPath()) {
-                    mRpc->getFreeSpaceForPath(directory);
-                    return;
-                }
-                if (directory == mRpc->serverSettings()->data().downloadDirectory) {
-                    mRpc->getDownloadDirFreeSpace();
-                    return;
-                }
-            }
-            freeSpaceLabel->hide();
-            freeSpaceLabel->clear();
-        };
+        mFreeSpaceLabel = new QLabel(this);
         QObject::connect(
             mAddTorrentParametersWidgets.downloadDirectoryWidget,
             &RemoteDirectorySelectionWidget::pathChanged,
             this,
-            [=, this] { getFreeSpace(mAddTorrentParametersWidgets.downloadDirectoryWidget->path()); }
+            [=, this] { onDownloadDirectoryPathChanged(mAddTorrentParametersWidgets.downloadDirectoryWidget->path()); }
         );
-        if (mRpc->serverSettings()->data().canShowFreeSpaceForPath()) {
-            QObject::connect(
-                mRpc,
-                &Rpc::gotFreeSpaceForPath,
-                this,
-                [=, this](const QString& path, bool success, long long bytes) {
-                    if (path == mAddTorrentParametersWidgets.downloadDirectoryWidget->path()) {
-                        if (success) {
-                            freeSpaceLabel->setText(
-                                //: %1 is a amount of free space in a directory, e.g. 1 GiB
-                                qApp->translate("tremotesf", "Free space: %1").arg(formatutils::formatByteSize(bytes))
-                            );
-                        } else {
-                            freeSpaceLabel->setText(qApp->translate("tremotesf", "Error getting free space"));
-                        }
-                        freeSpaceLabel->show();
-                    }
-                }
-            );
-        } else {
-            QObject::connect(mRpc, &Rpc::gotDownloadDirFreeSpace, this, [=, this](auto bytes) {
-                if (mAddTorrentParametersWidgets.downloadDirectoryWidget->path() ==
-                    mRpc->serverSettings()->data().downloadDirectory) {
-                    freeSpaceLabel->setText(
-                        qApp->translate("tremotesf", "Free space: %1").arg(formatutils::formatByteSize(bytes))
-                    );
-                    freeSpaceLabel->show();
-                }
-            });
-        }
-        getFreeSpace(mAddTorrentParametersWidgets.downloadDirectoryWidget->path());
+        onDownloadDirectoryPathChanged(mAddTorrentParametersWidgets.downloadDirectoryWidget->path());
         layout->insertRow(
             rowForWidget(layout, mAddTorrentParametersWidgets.downloadDirectoryWidget) + 1,
             nullptr,
-            freeSpaceLabel
+            mFreeSpaceLabel
         );
 
-        if (mMode == Mode::File) {
+        if (isAddingFile()) {
             mTorrentFilesView = new TorrentFilesView(mFilesModel, mRpc);
-            layout->insertRow(rowForWidget(layout, freeSpaceLabel) + 1, mTorrentFilesView);
+            layout->insertRow(rowForWidget(layout, mFreeSpaceLabel) + 1, mTorrentFilesView);
         } else {
             layout->addItem(new QSpacerItem(0, 0, QSizePolicy::Minimum, QSizePolicy::Expanding));
         }
@@ -287,47 +247,7 @@ namespace tremotesf {
         QObject::connect(mDialogButtonBox, &QDialogButtonBox::rejected, this, &AddTorrentDialog::reject);
         layout->addRow(mDialogButtonBox);
 
-        const auto updateUi = [=, this] {
-            const bool enabled = mRpc->isConnected() && (mFilesModel ? mFilesModel->isLoaded() : true);
-            if (enabled) {
-                // Update parameters which initial values depend on server state
-                const auto parameters = getAddTorrentParameters(mRpc);
-                mAddTorrentParametersWidgets.downloadDirectoryWidget->updatePath(parameters.downloadDirectory);
-                mAddTorrentParametersWidgets.startTorrentCheckBox->setChecked(parameters.startAfterAdding);
-            }
-
-            for (int i = 1, max = layout->count(); i < max; ++i) {
-                auto* const widget = layout->itemAt(i)->widget();
-                if (widget) {
-                    widget->setEnabled(enabled);
-                }
-            }
-
-            mAddTorrentParametersWidgets.startTorrentCheckBox->setEnabled(enabled);
-
-            if (enabled) {
-                if (messageWidget->isShowAnimationRunning()) {
-                    messageWidget->hide();
-                } else {
-                    messageWidget->animatedHide();
-                }
-                messageWidget->animatedHide();
-            } else if (mFilesModel && !mFilesModel->isLoaded()) {
-                messageWidget->setMessageType(KMessageWidget::Information);
-                //: Placeholder shown when torrent file is being read/parsed
-                messageWidget->setText(qApp->translate("tremotesf", "Loading"));
-                messageWidget->animatedShow();
-            } else {
-                messageWidget->setMessageType(KMessageWidget::Warning);
-                //: Server connection status
-                messageWidget->setText(qApp->translate("tremotesf", "Disconnected"));
-                messageWidget->animatedShow();
-            }
-
-            canAcceptUpdate();
-        };
-
-        QObject::connect(mRpc, &Rpc::connectedChanged, this, updateUi);
+        QObject::connect(mRpc, &Rpc::connectedChanged, this, &AddTorrentDialog::updateUi);
         QObject::connect(
             mAddTorrentParametersWidgets.downloadDirectoryWidget,
             &RemoteDirectorySelectionWidget::pathChanged,
@@ -335,35 +255,55 @@ namespace tremotesf {
             &AddTorrentDialog::canAcceptUpdate
         );
 
-        if (mFilesModel) {
-            QObject::connect(mFilesModel, &LocalTorrentFilesModel::loadedChanged, this, [=, this] {
-                if (mFilesModel->isSuccessfull()) {
-                    updateUi();
-                } else {
-                    auto messageBox = new QMessageBox(
-                        QMessageBox::Critical,
-                        //: Dialog title
-                        qApp->translate("tremotesf", "Error"),
-                        mFilesModel->errorString(),
-                        QMessageBox::Close,
-                        parentWidget()
-                    );
-                    messageBox->setAttribute(Qt::WA_DeleteOnClose);
-                    messageBox->show();
-                    close();
-                }
-            });
-        }
-
         // If call updateUi() right now and need to show messageWidget, it will be
         // shown without animation, because we are not visible yet
         // Call it at the next available event loop iteration
-        QTimer::singleShot(0, this, updateUi);
+        QTimer::singleShot(0, this, &AddTorrentDialog::updateUi);
+    }
+
+    void AddTorrentDialog::updateUi() {
+        const bool enabled = mRpc->isConnected() && (mFilesModel ? mFilesModel->isLoaded() : true);
+        if (enabled) {
+            // Update parameters which initial values depend on server state
+            const auto parameters = getAddTorrentParameters(mRpc);
+            mAddTorrentParametersWidgets.downloadDirectoryWidget->updatePath(parameters.downloadDirectory);
+            mAddTorrentParametersWidgets.startTorrentCheckBox->setChecked(parameters.startAfterAdding);
+        }
+
+        for (int i = 1, max = layout()->count(); i < max; ++i) {
+            auto* const widget = layout()->itemAt(i)->widget();
+            if (widget) {
+                widget->setEnabled(enabled);
+            }
+        }
+
+        mAddTorrentParametersWidgets.startTorrentCheckBox->setEnabled(enabled);
+
+        if (enabled) {
+            if (mMessageWidget->isShowAnimationRunning()) {
+                mMessageWidget->hide();
+            } else {
+                mMessageWidget->animatedHide();
+            }
+            mMessageWidget->animatedHide();
+        } else if (mFilesModel && !mFilesModel->isLoaded()) {
+            mMessageWidget->setMessageType(KMessageWidget::Information);
+            //: Placeholder shown when torrent file is being read/parsed
+            mMessageWidget->setText(qApp->translate("tremotesf", "Loading"));
+            mMessageWidget->animatedShow();
+        } else {
+            mMessageWidget->setMessageType(KMessageWidget::Warning);
+            //: Server connection status
+            mMessageWidget->setText(qApp->translate("tremotesf", "Disconnected"));
+            mMessageWidget->animatedShow();
+        }
+
+        canAcceptUpdate();
     }
 
     void AddTorrentDialog::canAcceptUpdate() {
         bool can = mRpc->isConnected() && (mFilesModel ? mFilesModel->isLoaded() : true);
-        if (mMode == Mode::Url && mTorrentLinkLineEdit->text().isEmpty()) {
+        if (mTorrentLinkTextField && mTorrentLinkTextField->document()->isEmpty()) {
             can = false;
         }
         if (mAddTorrentParametersWidgets.downloadDirectoryWidget->path().isEmpty()) {
@@ -378,6 +318,104 @@ namespace tremotesf {
         }
     }
 
+    void AddTorrentDialog::onDownloadDirectoryPathChanged(QString path) {
+        mFreeSpaceLabel->hide();
+        mFreeSpaceLabel->clear();
+        mFreeSpaceCoroutineScope.cancelAll();
+        if (!path.isEmpty()) {
+            mFreeSpaceCoroutineScope.launch(getFreeSpaceForPath(std::move(path)));
+        }
+    }
+
+    Coroutine<> AddTorrentDialog::getFreeSpaceForPath(QString path) {
+        const auto freeSpace = co_await mRpc->getFreeSpaceForPath(std::move(path));
+        if (freeSpace) {
+            mFreeSpaceLabel->setText(
+                //: %1 is a amount of free space in a directory, e.g. 1 GiB
+                qApp->translate("tremotesf", "Free space: %1").arg(formatutils::formatByteSize(*freeSpace))
+            );
+        } else {
+            mFreeSpaceLabel->setText(qApp->translate("tremotesf", "Error getting free space"));
+        }
+        mFreeSpaceLabel->show();
+    }
+
+    Coroutine<> AddTorrentDialog::parseTorrentFile() {
+        const auto& filePath = std::get<FileParams>(mParams).filePath;
+        try {
+            info().log("Parsing torrent file {}", filePath);
+            auto torrentFile = co_await runOnThreadPool(&tremotesf::parseTorrentFile, filePath);
+            info().log("Parsed, result = {}", torrentFile);
+            mTorrentFileInfoHashAndTrackers =
+                std::pair{std::move(torrentFile.infoHashV1), std::move(torrentFile.trackers)};
+            if (checkIfTorrentFileExists()) {
+                deleteTorrentFileIfEnabled();
+                close();
+                co_return;
+            }
+            co_await mFilesModel->load(std::move(torrentFile));
+            updateUi();
+        } catch (const bencode::Error& e) {
+            warning().logWithException(e, "Failed to parse torrent file {}", filePath);
+            showTorrentParsingError(bencodeErrorString(e.type()));
+            close();
+        }
+    }
+
+    void AddTorrentDialog::showTorrentParsingError(const QString& errorString) {
+        auto* const messageBox = new QMessageBox(
+            QMessageBox::Critical,
+            //: Dialog title
+            qApp->translate("tremotesf", "Error"),
+            errorString,
+            QMessageBox::Close,
+            parentWidget()
+        );
+        messageBox->setAttribute(Qt::WA_DeleteOnClose);
+        messageBox->show();
+    }
+
+    void AddTorrentDialog::parseMagnetLinksAndCheckIfTorrentsExist(QStringList& urls) {
+        auto toErase = std::ranges::remove_if(urls, [&](const QString& url) {
+            try {
+                info().log("Parsing {} as a magnet link", url);
+                auto magnetLink = tremotesf::parseMagnetLink(QUrl(url));
+                info().log("Parsed, result = {}", magnetLink);
+                auto* const torrent = mRpc->torrentByHash(magnetLink.infoHashV1);
+                if (torrent) {
+                    askForMergingTrackers(torrent, std::move(magnetLink.trackers), parentWidget());
+                    return true;
+                }
+            } catch (const std::runtime_error& e) {
+                warning().logWithException(e, "Failed to parse {} as a magnet link", url);
+            }
+            return false;
+        });
+        if (!toErase.empty()) {
+            urls.erase(toErase.begin(), toErase.end());
+        }
+    }
+
+    bool AddTorrentDialog::checkIfTorrentFileExists() {
+        if (!mTorrentFileInfoHashAndTrackers.has_value()) return false;
+        auto& [infoHashV1, trackers] = *mTorrentFileInfoHashAndTrackers;
+        auto* const torrent = mRpc->torrentByHash(infoHashV1);
+        if (torrent) {
+            askForMergingTrackers(torrent, std::move(trackers), parentWidget());
+            return true;
+        }
+        return false;
+    }
+
+    void AddTorrentDialog::deleteTorrentFileIfEnabled() {
+        if (isAddingFile() && mAddTorrentParametersWidgets.deleteTorrentFileGroupBox->isChecked()) {
+            deleteTorrentFile(
+                std::get<FileParams>(mParams).filePath,
+                mAddTorrentParametersWidgets.moveTorrentFileToTrashCheckBox->isChecked()
+            );
+        }
+    }
+
     void AddTorrentDialog::AddTorrentParametersWidgets::reset(Rpc* rpc) const {
         const auto initialParameters = getInitialAddTorrentParameters(rpc);
         downloadDirectoryWidget->updatePath(initialParameters.downloadDirectory);
