@@ -5,7 +5,16 @@
 #ifndef TREMOTESF_ADDTORRENTDIALOG_H
 #define TREMOTESF_ADDTORRENTDIALOG_H
 
+#include <optional>
+#include <set>
+#include <utility>
+#include <variant>
+#include <vector>
+
 #include <QDialog>
+#include <QStringList>
+
+#include "coroutines/scope.h"
 #include "ui/savewindowstatedispatcher.h"
 
 class QCheckBox;
@@ -15,6 +24,9 @@ class QFormLayout;
 class QGroupBox;
 class QLabel;
 class QLineEdit;
+class QPlainTextEdit;
+
+class KMessageWidget;
 
 namespace tremotesf {
     class LocalTorrentFilesModel;
@@ -26,9 +38,15 @@ namespace tremotesf {
         Q_OBJECT
 
     public:
-        enum class Mode { File, Url };
+        struct FileParams {
+            QString filePath;
+        };
 
-        explicit AddTorrentDialog(Rpc* rpc, const QString& url, Mode mode, QWidget* parent = nullptr);
+        struct UrlParams {
+            QStringList urls;
+        };
+
+        explicit AddTorrentDialog(Rpc* rpc, std::variant<FileParams, UrlParams> params, QWidget* parent = nullptr);
 
         void accept() override;
 
@@ -43,25 +61,46 @@ namespace tremotesf {
             void saveToSettings() const;
         };
 
-        static AddTorrentParametersWidgets createAddTorrentParametersWidgets(Mode mode, QFormLayout* layout, Rpc* rpc);
+        static AddTorrentParametersWidgets
+        createAddTorrentParametersWidgets(bool forTorrentFile, QFormLayout* layout, Rpc* rpc);
 
     private:
+        bool isAddingFile() const;
+
         void setupUi();
+        void updateUi();
         void canAcceptUpdate();
         void saveState();
 
+        void onDownloadDirectoryPathChanged(QString path);
+        Coroutine<> getFreeSpaceForPath(QString path);
+
+        Coroutine<> parseTorrentFile();
+        void showTorrentParsingError(const QString& errorString);
+
+        void parseMagnetLinksAndCheckIfTorrentsExist(QStringList& urls);
+        bool checkIfTorrentFileExists();
+
+        void deleteTorrentFileIfEnabled();
+
         Rpc* mRpc;
-        QString mUrl;
-        Mode mMode;
+        std::variant<FileParams, UrlParams> mParams;
 
         LocalTorrentFilesModel* mFilesModel{};
+        CoroutineScope mParseTorrentFileCoroutineScope{};
+        std::optional<std::pair<QString, std::vector<std::set<QString>>>> mTorrentFileInfoHashAndTrackers{};
 
-        QLineEdit* mTorrentLinkLineEdit{};
+        KMessageWidget* mMessageWidget{};
+        QLineEdit* mTorrentFilePathTextField{};
+        QPlainTextEdit* mTorrentLinkTextField{};
+        QLabel* mFreeSpaceLabel{};
         TorrentFilesView* mTorrentFilesView{};
         AddTorrentParametersWidgets mAddTorrentParametersWidgets{};
 
         QDialogButtonBox* mDialogButtonBox{};
 
+        CoroutineScope mFreeSpaceCoroutineScope{};
+
         SaveWindowStateHandler mSaveStateHandler{this, [this] { saveState(); }};
     };
 }
